/* **************************************************  */
/* automatizovana expozice zatmeni Slunce              */
/* hodnota expozicniho casu se musi zadat dle          */
/* tabulky v seznamu v eosvaluetitles.h                */

/* upraveno podle EOSRemote_glider                     */

/*             finalni verze */   

#include <Wire.h>
#include <LiquidCrystal_I2C.h>

#include <usbhub.h>

#include <ptp.h>
#include <canoneos.h>
#include <valuelist.h>
#include <eosvaluetitles.h>
#include <eoseventparser.h>

#include "eoseventhandlers.h"

#define EEP_APERTURE_LIST_OFFSET    0
#define EEP_APERTURE_LIST_SIZE      32

#define EEP_SHTSPEED_LIST_OFFSET    (EEP_APERTURE_LIST_OFFSET + EEP_APERTURE_LIST_SIZE)
#define EEP_SHTSPEED_LIST_SIZE      64

#define EEP_WBALANCE_LIST_OFFSET    (EEP_SHTSPEED_LIST_OFFSET + EEP_SHTSPEED_LIST_SIZE)
#define EEP_WBALANCE_LIST_SIZE      12

#define EEP_PICSTYLE_LIST_OFFSET    (EEP_WBALANCE_LIST_OFFSET + EEP_WBALANCE_LIST_SIZE)
#define EEP_PICSTYLE_LIST_SIZE      12

#define EEP_EXPOCOR_LIST_OFFSET     (EEP_PICSTYLE_LIST_OFFSET + EEP_PICSTYLE_LIST_SIZE)
#define EEP_EXPOCOR_LIST_SIZE       48

#define EEP_ISO_LIST_OFFSET         (EEP_EXPOCOR_LIST_OFFSET + EEP_EXPOCOR_LIST_SIZE)
#define EEP_ISO_LIST_SIZE           8

EEPROMByteList          vlAperture(EEP_APERTURE_LIST_OFFSET, EEP_APERTURE_LIST_SIZE);
EEPROMByteList          vlShutterSpeed(EEP_SHTSPEED_LIST_OFFSET, EEP_SHTSPEED_LIST_SIZE);
EEPROMByteList          vlWhiteBalance(EEP_WBALANCE_LIST_OFFSET, EEP_WBALANCE_LIST_SIZE);
EEPROMByteList          vlPictureStyle(EEP_PICSTYLE_LIST_OFFSET, EEP_PICSTYLE_LIST_SIZE);
EEPROMByteList          vlIso(EEP_ISO_LIST_OFFSET, EEP_ISO_LIST_SIZE);
EEPROMByteList          vlExpCompensation(EEP_EXPOCOR_LIST_OFFSET, EEP_EXPOCOR_LIST_SIZE);

/* deklarace displeje */
/* z GME s I2C radicem */
LiquidCrystal_I2C lcd(0x3F, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);

int LedRed = 6;       // pin D6 na Arduinu
int LedGreen = 5;     // pin D7 na Arduinu
int ShutterPin = 2;   // pin D2 (Interrupt1) na Arduinu

class CamStateHandlers : public EOSStateHandlers
{
      enum CamStates { stInitial, stDisconnected, stConnected };
      CamStates stateConnected;

public:
      CamStateHandlers() : stateConnected(stInitial)
      {
      };

      virtual void OnDeviceDisconnectedState(PTP *ptp);
      virtual void OnDeviceInitializedState(PTP *ptp);
};

CamStateHandlers    CamStates;
USB                 Usb;
USBHub              Hub1(&Usb);
CanonEOS            Eos(&Usb, &CamStates);

uint8_t  dpMode         = 0;
uint8_t  dpAperture     = 0;
uint8_t  dpShutterSpeed = 0;
uint8_t  dpWb           = 0;
uint8_t  dpPStyle       = 0;
uint8_t  dpIso          = 0;
uint8_t  dpExpComp      = 0;
uint8_t  dEV            = 1;
uint8_t  HDR            = 5;
int  aHDR           = -1;
bool  HDRon           = false;
int CurPos = 1;

bool  LV = false;
double cas;
uint8_t dpS0 = 0;
int index;
int pauza;
int flag = 0;
int WhatIsShooting;
int IsShooting;
int EosOn;
unsigned long int timelapse;


void RedOnGreenOff() {
    digitalWrite(LedRed, HIGH);
    digitalWrite(LedGreen, LOW);  
    EosOn=0;
}

void RedOffGreenOn() {
    digitalWrite(LedRed, LOW);
    digitalWrite(LedGreen, HIGH);  
    EosOn=1;
}


void CamStateHandlers::OnDeviceDisconnectedState(PTP *ptp)
{
    if (stateConnected == stConnected || stateConnected == stInitial)
    {
        stateConnected = stDisconnected;
        E_Notify(PSTR("Camera disconnected.\r\n"),0x80);
        lcd.setCursor(0,0);
        lcd.print(" EOS Solar Eclipse ");
        lcd.setCursor(0,1);
        lcd.noBlink();
        lcd.print("  EOS disconnected. ");
        RedOnGreenOff();
        lcd.setCursor(0,2);
        lcd.print("                    ");
        lcd.setCursor(0,3);
        lcd.print("                    ");
        if (stateConnected == stConnected) {
           RedOffGreenOn();
           lcd.setCursor(0,0);
           lcd.print(" EOS Solar Eclipse ");
           lcd.setCursor(0,1);
           lcd.print("   EOS connected.   ");
           lcd.setCursor(0,2);
           lcd.print("                    ");
           lcd.setCursor(0,3);
           lcd.print("                    ");
           delay(1000);
           lcd.setCursor(0,1);
           lcd.print("                    ");
           lcd.blink();
           lcd.setCursor(0,1);CurPos=1;
        }
    }
}

void CamStateHandlers::OnDeviceInitializedState(PTP *ptp)
{
    static uint32_t  next_time = 0;

    if (stateConnected == stDisconnected || stateConnected == stInitial)
    {
        stateConnected = stConnected;
           RedOffGreenOn();
           lcd.setCursor(0,0);
           lcd.print(" EOS Solar Eclipse ");
           lcd.setCursor(0,1);
           lcd.print("   EOS connected.   ");
           lcd.setCursor(0,2);
           lcd.print("                    ");
           lcd.setCursor(0,3);
           lcd.print("                    ");
           delay(1000);
           lcd.setCursor(0,1);
           lcd.print("                    ");
           lcd.blink();
           lcd.setCursor(0,1);CurPos=1;
    }
    uint32_t time_now = millis();

    if (time_now >= next_time)
    {
        next_time = time_now + 300;

        EosEventHandlers  hnd;
        EOSEventParser    prs(&hnd);
        Eos.EventCheck(&prs);

    }
}

void setup() {
    pinMode(ShutterPin, OUTPUT);
    digitalWrite(ShutterPin, LOW);

    Serial.begin(115200);
    
    EosOn=0;
    /* zablikani dvojbarevne LEDky */
    
    pinMode(LedRed,OUTPUT);
    digitalWrite(LedRed, HIGH);
    pinMode(LedGreen,OUTPUT);
    digitalWrite(LedGreen, LOW);
    delay(1000);
    digitalWrite(LedRed, LOW);
    digitalWrite(LedGreen, HIGH);
    delay(1000);
    digitalWrite(LedRed, HIGH);
    digitalWrite(LedGreen, LOW);
    
    if (Usb.Init() == -1)
        Serial.println("OSC did not start.");

    delay( 200 );

    Serial.println("Start");
    
   lcd.begin(20,4);
   lcd.home();
   lcd.setCursor(0,0);
   lcd.print(" EOS Solar Eclipse ");
   lcd.setCursor(0,1);
   lcd.print("* based on Arduino *");
   lcd.setCursor(0,2);
   lcd.print("** version 0.7e **");
   lcd.setCursor(0,3);
   lcd.print("*** (c) StanDa ***");
   delay(1500);
   lcd.setCursor(0,1);
   lcd.print("* switch EOS to the *");
   lcd.setCursor(0,2);
   lcd.print("** CONTINUOUS MODE **");
   lcd.setCursor(0,3);
   lcd.print("                     ");
   delay(5000);
   Serial.begin(115200);
   lcd.clear();
   lcd.setCursor(0,0);
   lcd.print(" EOS Solar Eclipse ");
   lcd.setCursor(0,2);
   lcd.blink();
   lcd.setCursor(1,0);
   delay(10000);

  /* Interrupt for pushbutton */

  attachInterrupt(1,PushButton,RISING);
  IsShooting=0;
  timelapse=0;
}

void PushButton() {
     detachInterrupt(1);
     delayMicroseconds(250000L); // 0.25s pause (debouncing button)
        if (IsShooting==0) {
          IsShooting=1;
       } else {
          IsShooting=0;
          WhatIsShooting++;
          ReleaseShutter();
       }
       attachInterrupt(1,PushButton,RISING);
}

void PressShutter() {
      digitalWrite(ShutterPin, HIGH);
}

void ReleaseShutter() {
      digitalWrite(ShutterPin, LOW);
}

void loop() {

  Usb.Task();
  Serial.println(dpShutterSpeed);
  Serial.print("WhatIsShooting = ");
  Serial.println(WhatIsShooting);
  Serial.print("  IsShooting = ");
  Serial.println(IsShooting);
  delay(500);
  
  /* zde je mozno nastavit, s cim se zacne fotit                        */
  /* WhatIsShooting = 0   zacne se fotit castecne zatmeni casosberem    */
  /* WhatIsShooting = 1   zacne se fotit Bayliho perly PRED uplnou fazi */
  /* WhatIsShooting = 2   zacne se fotit uplna faze                     */
  /* WhatIsShooting = 3   zacne se fotit Bayliho perly PO uplne fazi    */
  /* WhatIsShooting = 4   zacne se fotit castecne zatmeni casosberem PO */
  
  // WhatIsShooting=0;
  
  if (EosOn==1) {
  switch (WhatIsShooting) {
    case 0: lcd.setCursor(0,2);
            lcd.print("  Press Button for ");
            lcd.setCursor(0,3);
            if (IsShooting==0) {
               lcd.print("START Partial Eclps. ");
               Eos.SetProperty(EOS_DPC_ShutterSpeed,0x98);  // 1/4000 s
               delay(250);  
               Eos.SetProperty(EOS_DPC_Aperture,0x30);  // 5.6 
               delay(250);                 
               Eos.SetProperty(EOS_DPC_Iso,0x48);  // 100
               delay(250);                 
               Eos.SetProperty(EOS_DPC_ShootingMode,3);  // Manual Mode
               delay(250);    
            } else {                            
               lcd.print("STOP Partial Eclps. ");
               PartialEclipse(5);    // each 5 sec take a picture               
            }   
            lcd.setCursor(0,2);
            break;
    case 1: lcd.setCursor(0,2);
            Eos.SetProperty(EOS_DPC_ShutterSpeed,0x98);  // 1/4000 s
            delay(250);   
            Eos.SetProperty(EOS_DPC_Aperture,0x30);  // 5.6 
            delay(250);                 
            Eos.SetProperty(EOS_DPC_Iso,0x48);  // 100
            delay(250);                 
            lcd.print("  Press Button for  ");
            lcd.setCursor(0,3);
            if (IsShooting==0) {
              lcd.print("START Baily's beads ");
            } else {
              lcd.print(" STOP Baily's beads ");
              while (IsShooting==1) {
                PressShutter();delay(100);ReleaseShutter();
                delay(500);   // 2 snapshots per second without LV, 1 per sec with LV on               
              }
            }  
            lcd.setCursor(0,2);
            break;
    case 2: lcd.setCursor(0,2);
            lcd.print("  Press Button for ");
            lcd.setCursor(0,3);
            if (IsShooting==0) {
               lcd.print("START Total Eclipse ");
            } else {
               lcd.setCursor(0,2);
               lcd.print("  Total Eclipse is  ");                
               lcd.setCursor(0,3);
               lcd.print(" being photographed."); 
               TotalEclipse();
            }   
            lcd.setCursor(0,2);
            break;
    case 3: lcd.setCursor(0,2);
            lcd.print("  Press Button for ");
            lcd.setCursor(0,3);
            if (IsShooting==0) {
              Eos.SetProperty(EOS_DPC_ShutterSpeed,0x98);  // 1/4000 s
              delay(250);  
              Eos.SetProperty(EOS_DPC_Aperture,0x30);  // 5.6 
              delay(250);                 
              Eos.SetProperty(EOS_DPC_Iso,0x48);  // 100
              delay(250);                 
              lcd.print("START Baily's beads ");
            } else {
              lcd.print(" STOP Baily's beads ");
              while (IsShooting==1) {
                PressShutter();delay(100);ReleaseShutter();
                delay(500);   // 2 snapshots per second without LV, 1 per sec with LV on                               
              }
            }  
            lcd.setCursor(0,2);
            break;            
    case 4: lcd.setCursor(0,2);
            lcd.print("  Press Button for ");
            lcd.setCursor(0,3);
            if (IsShooting==0) {
               Eos.SetProperty(EOS_DPC_ShutterSpeed,0x98);  // 1/4000 s
               delay(250);  
               lcd.print("START Partial Eclps. ");
            } else {
               Eos.SetProperty(EOS_DPC_ShutterSpeed,0x98);  // 1/4000 s
               delay(250);  
               Eos.SetProperty(EOS_DPC_Aperture,0x30);  // 5.6 s
               delay(250);                 
               Eos.SetProperty(EOS_DPC_Iso,0x48);  // 100
               delay(250);      
               PartialEclipse(5);    // each 5 sec take a picture
               lcd.print("STOP Partial Eclps. ");               
            }   
            lcd.setCursor(0,2);
            break;
    case 5: lcd.setCursor(0,2);
            lcd.print("    Solar Eclipse    ");
            lcd.setCursor(0,3);
            lcd.print("    Photographed!    ");
            break;
  } // end switch

  } // end EosOn
 /*
 if (dpShutterSpeed>=12) {
     dpShutterSpeed+=4;
        Eos.SetProperty(EOS_DPC_ShutterSpeed,dpShutterSpeed);     
     if (dpShutterSpeed==160) {
       Eos.Capture();delay(1000);
       dpShutterSpeed=12;
       Eos.SetProperty(EOS_DPC_ShutterSpeed,dpShutterSpeed);     

     }   
  }
 */
}

/* **************************************************************************
 *                                                                          *
 *                         PARTIAL ECLIPSE TIMELAPSE                        *
 *                                                                          *
 * ************************************************************************** */
 
void PartialEclipse(int interval) {
     uint8_t T;
     T=0x98;
     Eos.SetProperty(EOS_DPC_ShutterSpeed,T);  // 1/4000 s
     delay(250); 
     if (WhatIsShooting==0 || WhatIsShooting==4 && IsShooting==1) {
         if ((millis()-timelapse)>interval*1e3) { 
              Eos.Capture();
              timelapse=millis();
         }    
         Usb.Task();         
     }
}

/* **************************************************************************
 *                                                                          *
 *                         BAILY'S BEADS PHOTOGRAPHY                        *
 *                                                                          *
 * ************************************************************************** */

void BeadsEclipse() {
     uint8_t T;
     T=0x98;
     Eos.SetProperty(EOS_DPC_ShutterSpeed,T);  // 1/4000 s
     delay(250); 
     while (1) {
     PressShutter();delay(100);ReleaseShutter();
     delay(1000);
     }
}

/* **************************************************************************
 *                                                                          *
 *                           TOTAL ECLIPSE PHOTOGRAPHY                      *
 *                                                                          *
 * ************************************************************************** */


void TotalEclipse() {
     uint8_t T;
     int i;
     int waiting_time;
     float delay_time0;
     
     T=0x98;   // 1/4000s
     delay_time0=1.0/4000.0;
     Eos.SetProperty(EOS_DPC_ShutterSpeed,T);  // 1/4000 s
     delay(250);
     for (i=0;i<15;i++) {
        Eos.Capture();  
        if (delay_time0<1) {
          waiting_time=2500;
        } else {
          waiting_time=delay_time0*1000+2500;
        }
        delay(waiting_time);   
        T=T-8;  // T->T*2 (shift of -1EV)
        delay_time0=delay_time0*2.0;
        Eos.SetProperty(EOS_DPC_ShutterSpeed,T);  // new shutterspeed
        delay(250);
        Usb.Task();
        delay(250);
     }
     IsShooting=0;
     WhatIsShooting++;
}


