#include "eoseventhandlers.h"
#include <eosvaluetitles.h>

#include <Wire.h>
#include <LiquidCrystal_I2C.h>

extern LiquidCrystal_I2C              lcd;
extern EEPROMByteList          vlAperture;
extern EEPROMByteList          vlShutterSpeed;
extern EEPROMByteList          vlWhiteBalance;
extern EEPROMByteList          vlPictureStyle;
extern EEPROMByteList          vlExpCompensation;
extern EEPROMByteList          vlIso;

extern uint8_t  dpMode;
extern uint8_t  dpAperture;
extern uint8_t  dpShutterSpeed;
extern uint8_t  dpWb;
extern uint8_t  dpPStyle;
extern uint8_t  dpIso;
extern uint8_t  dpExpComp;
extern uint8_t  dEV;
extern uint8_t  HDR;

extern double cas;

void EosEventHandlers::OnPropertyChanged(const EOSEvent *evt)
{
    char s[30];
    char *ptr;
    uint8_t index;
    int tt;
    char *msg;
    switch (evt->propCode)
    {
    case EOS_DPC_Aperture:
        dpAperture = (uint8_t) evt->propValue;
        E_Notify(PSTR("F:"),0x80);
        E_Notify((char*)FindTitle<VT_APERTURE, VT_APT_TEXT_LEN>(VT_APT_COUNT, ApertureTitles, dpAperture),0x80);
        E_Notify(PSTR("\r\n"),0x80);
                lcd.setCursor(7,1);  // zacatek 2.radku
        lcd.print("F: ");
        index = 0;
        msg=(char*)FindTitle<VT_APERTURE, VT_APT_TEXT_LEN>(VT_APT_COUNT, ApertureTitles, dpAperture);
        while (s[index] = pgm_read_byte(msg++)) {
          index++;
        };
        tt=pgm_read_byte_near(ShutterSpeedTitles + 6);
        lcd.setCursor(9,1);  // zacatek 2.radku
        lcd.print(s);    
        break;
    case EOS_DPC_ShutterSpeed:
        dpShutterSpeed = (uint8_t) evt->propValue;
        E_Notify(PSTR("T:"),0x80);
        E_Notify((char*)FindTitle<VT_SHSPEED, VT_SHSPEED_TEXT_LEN>(VT_SHSPEED_COUNT, ShutterSpeedTitles, dpShutterSpeed),0x80);
        E_Notify(PSTR("\r\n"),0x80); 
        lcd.setCursor(0,1);  // zacatek 2.radku
        lcd.print("T:");
        index = 0;
        msg=(char*)FindTitle<VT_SHSPEED, VT_SHSPEED_TEXT_LEN>(VT_SHSPEED_COUNT, ShutterSpeedTitles, dpShutterSpeed);
        while (s[index] = pgm_read_byte(msg++)) {
          index++;
        };
        tt=pgm_read_byte_near(ShutterSpeedTitles + 6);
        lcd.setCursor(2,1);  // zacatek 2.radku
        lcd.print(s);
//        lcd.print("  ");
        ptr=strchr(s,'"');
        Serial.println(s);
        Serial.println(ptr);     
        if (ptr!=NULL) { 
           s[strlen(s)-strlen(ptr)]='.';
           cas=atof(s);
        } else {
           cas=1/atof(s);
        }   
        Serial.println(cas);
        break;        
    case EOS_DPC_ShootingMode:
        dpMode = (uint8_t) evt->propValue;
        E_Notify(PSTR("Mode:"),0x80);
        E_Notify((char*)FindTitle<VT_MODE, VT_MODE_TEXT_LEN>(VT_MODE_COUNT, ModeTitles, dpMode),0x80);
        E_Notify(PSTR("\r\n"),0x80);
//        lcd.setCursor(10,3);  // zacatek 4.radku
//        lcd.print("Mode:");
        index = 0;
        msg=(char*)FindTitle<VT_MODE, VT_MODE_TEXT_LEN>(VT_MODE_COUNT, ModeTitles, dpMode);
        while (s[index] = pgm_read_byte(msg++)) {
          index++;
        };
        tt=pgm_read_byte_near(ShutterSpeedTitles + 6);
//        lcd.setCursor(16,3);  // zacatek 2.radku
//        lcd.print(s);

        
        break;
    case EOS_DPC_Iso:
        dpIso = (uint8_t) evt->propValue;
        E_Notify(PSTR("ISO:"),0x80);
        E_Notify((char*)FindTitle<VT_ISO, VT_ISO_TEXT_LEN>(VT_ISO_COUNT, IsoTitles, dpIso),0x80);
        E_Notify(PSTR("\r\n"),0x80); 
        lcd.setCursor(13,1);  // zacatek 4.radku
        lcd.print("IS:");
        index = 0;
        msg=(char*)FindTitle<VT_ISO, VT_ISO_TEXT_LEN>(VT_ISO_COUNT, IsoTitles, dpIso);
        while (s[index] = pgm_read_byte(msg++)) {
          index++;
        };
        tt=pgm_read_byte_near(ShutterSpeedTitles + 6);
        lcd.setCursor(16,1);  // 6.pozice 4.radku
        lcd.print(s);
        break;
    };
//    lcd.setCursor(10,2);  // 6.pozice 3.radku
//    sprintf(s,"dEV=%d",dEV);
//    lcd.print(s);
//    lcd.setCursor(10,1);  // 6.pozice 2.radku
//    sprintf(s,"HDR=%d",HDR);
//    lcd.print(s);
}

void EosEventHandlers::OnAcceptedListSize(const EOSEvent *evt, const uint16_t size)
{
    switch (evt->propCode)
    {
    case EOS_DPC_Aperture:
        vlAperture.SetSize((uint8_t)size);
        break;
    case EOS_DPC_ShutterSpeed:
        vlShutterSpeed.SetSize((uint8_t)size);
        break;
    case EOS_DPC_WhiteBalance:
        vlWhiteBalance.SetSize((uint8_t)size);
        break;
    case EOS_DPC_PictureStyle:
        vlPictureStyle.SetSize((uint8_t)size);
        break;
    case EOS_DPC_Iso:
        vlIso.SetSize((uint8_t)size);
        break;
    case EOS_DPC_ExposureCompensation:
        vlExpCompensation.SetSize((uint8_t)size);
        break;
    };
}

void EosEventHandlers::OnPropertyValuesAccepted(const EOSEvent *evt, const uint16_t index, const uint32_t &val)
{
    switch (evt->propCode)
    {
    case EOS_DPC_Aperture:
        vlAperture.Set(index, (uint8_t)val);
        break;
    case EOS_DPC_ShutterSpeed:
        vlShutterSpeed.Set(index, (uint8_t)val);
        break;
    case EOS_DPC_WhiteBalance:
        vlWhiteBalance.Set(index, (uint8_t)val);
        break;
    case EOS_DPC_PictureStyle:
        vlPictureStyle.Set(index, (uint8_t)val);
        break;
    case EOS_DPC_ExposureCompensation:
        vlExpCompensation.Set(index, (uint8_t)val);
        break;
    case EOS_DPC_Iso:
        vlIso.Set(index, (uint8_t)val);
        break;
    } // switch (evt->propCode)
}



